@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.29  -- Rateup
+ MRTG 2.10.6  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -35,13 +35,6 @@
 /* VC++ does not have unistd.h */
 #ifndef WIN32
   #include "../config.h"
-  /* maybe this will make rateup.c compile on HPUX */
-  #if !defined(HAVE_STRTOLL) && defined(HAVE___STRTOLL)
-     #ifdef HAVE_INTTYPES_H
-       #include <inttypes.h>
-     #endif
-     #define strtoll __strtoll
-  #endif
   #include <unistd.h>
 #endif
 #include <limits.h>
@@ -61,10 +54,18 @@
 /* MSVCRT.DLL does not know %ll in printf */ 
 #ifdef __MINGW32_VERSION
 #define LLD "%I64d"
-#else
-#define LLD "%lld"
 #endif
 
+#ifdef __EMX__  /* OS/2 */
+#define strtoll _strtoll
+#define LLD "%Ld" /* EMX lib use %Ld for long long */
+#endif
+
+#ifndef LLD
+#define  LLD "%lld"
+#endif
+
+
 /* WATCOM C/C++ 10.6 under Win95/NT */
 /* VC++ 6.0 under Win95/NT */
 #if defined(__WATCOMC__) || defined(WIN32)
@@ -77,7 +78,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.29";
+char *VERSION = "2.10.6";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -91,12 +92,17 @@ short options = 0;
 #define OPTION_NOARROW		0x0020	/* noarrow */
 #define OPTION_NO_I		0x0040	/* ignore 'I' (first) variable */
 #define OPTION_NO_O		0x0080	/* ignore 'O' (second) variable */
+#define OPTION_PNGDATE		0x0100	/* show date & time in graph */
+#define OPTION_PRINTROUTER		0x0200	/* show title in graph */
 
 time_t NOW;
 
 char *short_si[] = {"","k","M","G","T"};
 char *longup = NULL;
 char *shortup = NULL;
+char *pngtitle = NULL;
+char *pngdate = NULL;
+char *rtimezone = NULL;
 char weekformat = 'W';			/* strftime() fmt char for week #  */
 
 #define DAY_COUNT (600)			/* 400 samples is 33.33 hours */
@@ -237,12 +243,14 @@ int peak;
     double inr, outr,muli, interval;
     time_t now,onow,nextnow;
     struct tm tm2, *tm=&tm2;
+    struct tm tm3, *tmtz=&tm3;
     char **graph_label;
     char ylab[30];
     /* scaling helpers */
     long long maxv_q;
     long long valsamp,maxin,maxout,digits,maxpercent=0;
-    long long sca_ten,sca_hun,nmax_q;
+    long long sca_ten,sca_hun;
+    double nmax_q;
     double avmxin,avmxout, avin, avout,latestout=0,latestin=0,nmax,avpercent=0,latestpercent=0;
     double nex_ten,nex_hun,nex_rnd;
     double sca_max_q,dummy;
@@ -261,6 +269,8 @@ int peak;
 #define xtr(x) (long)((growright) ? (maxx*xscale+81-((x)*xscale)) : (81+((x)*xscale))) 
     /* ################################################# */ 
 
+
+
     /* GD LIB declarations */
     /* Declare the image */
     gdImagePtr graph,brush_out,brush_outm,brush_outp;
@@ -660,6 +670,38 @@ int peak;
 		  gdBrushed);
    }
  
+   /* print the graph title */ 
+     if ( pngtitle != NULL ) {	
+       gdImageString(graph, gdFontSmall,81, 1,
+         (unsigned char*)pngtitle, i_grid);
+     } else {
+       if ( options & OPTION_PRINTROUTER ) {
+        gdImageString(graph, gdFontSmall,81, 1,
+         (unsigned char*)router, i_grid);
+       }
+     }
+  
+   /* print the date & time */ 
+    if (   options & OPTION_PNGDATE  ) {	 
+      pngdate = (char *)calloc(1,100);
+	  tmtz=localtime(&NOW);
+	  if ( rtimezone == NULL ) {
+	  	strftime(pngdate, 18, "%b %d %Y %H:%M", tmtz); 
+      } else  {
+	  	strftime(pngdate, 19, "%b %d %Y %H:%M ", tmtz); 
+		if ( ((size_t) strlen(pngdate)+strlen(rtimezone)) < 100 ) {
+	      strcat(pngdate, rtimezone); 
+        } else {
+			fprintf(stderr,"Rateup ERROR: date + timezone string too long\n");
+            exit(1);
+		}
+
+      }
+	  gdImageString(graph, gdFontSmall,
+              xtr(maxx)-gdFontSmall->w*strlen(pngdate), 1,
+		      (unsigned char*)pngdate, i_grid);
+ 	} 
+  
     /* draw the graph border */
     gdImageRectangle(graph,xtr(0),ytr(0),xtr(maxx),ytr(maxy),i_grid);
     
@@ -774,6 +816,7 @@ int peak;
     gdImageDestroy(brush_outp);
     free(lhist);
     free(graph_label);
+
 }
 
 	    
@@ -800,20 +843,22 @@ char *a,*b;
     r1[1] = 0;	/* Null terminate result */
     c = 0;
     for (x=0; x<m; x++) {
-	if (a1 >= a && b1 >= b) {
-	    *r1 = ((*a1 - c) - *b1) + '0';
-	} else if (a1 >= a) {
-	    *r1 = (*a1 - c);
-	} else {
-	    *r1 = ('0' - *b1 - c) + '0';
-	}
-	if (*r1 < '0') {
-	    *r1 += 10; 
-	    c=1;
-	} else {
-	    c=0;
-	}
-	a1--;b1--;r1--;
+        /* we want to avoid reading off the edge of the string */
+        char save_a,save_b;
+        save_a = ( a1 >= a) ? *a1 : '0';
+        save_b = ( b1 >= b) ? *b1 : '0';
+        *r1 = save_a - save_b - c + '0';
+        if (*r1 < '0') {
+            *r1 += 10;
+            c=1;
+        } else
+          if (*r1 > '9') { /* 0 - 10 */
+            *r1 -= 10;
+            c=1;            
+          } else {
+            c=0;
+        }
+        a1--;b1--;r1--;
     }
     if (c) {
 	r1 = &res[m+1];
@@ -840,12 +885,14 @@ char *file;
     struct HISTORY *hist;
     long long rd[5];
     time_t cur;
+    long lasttime;    
 
     if ((fi = fopen(file,"r")) != NULL) {
-	if (fscanf(fi,"%ld %s %s\n",(long int *)&last.time,&last.in[0],&last.out[0]) != 3){
+	if (fscanf(fi,"%ld %s %s\n",&lasttime,&last.in[0],&last.out[0]) != 3){
            fprintf(stderr,"Read Error: File %s lin 1\n",file);
 	   retcode = 1;
 	}
+	last.time = lasttime;
         cur = last.time;
 	x = histvalid=0;
 	hist = history;
@@ -1275,6 +1322,10 @@ char **argv;
 		       options &= ~OPTION_NOBORDER;
 		       used = 1;
 		       break;
+	 	   case 'd':   /* Print date in image */
+		       options |= OPTION_PNGDATE;
+		       used = 1;
+		       break;
 	 	   case 'i':   /* Do not graph the I variable */
 		       options |= OPTION_NO_I;
 		       used = 1;
@@ -1290,6 +1341,9 @@ char **argv;
 		   case 'O':   /* Graph the O variable */
 		       options &= ~OPTION_NO_O;
 		       used = 1;
+		   case 'p':   /* print router name in image */
+		       options |= OPTION_PRINTROUTER;
+		       used = 1;
 		       break;
                    case 't':   /* Transparent Image */
                        options |= OPTION_TRANSPARENT;
@@ -1430,8 +1484,13 @@ char **argv;
                 kMGcopy = calloc(strlen(argv[argi+1])+1, sizeof(char));               
 		used = 2;
 		break;
-            case 'l':  /* YLegend - rewritten by Oliver Haid */
+	    case 'Z':	/* Timezone name */
+		rtimezone = argv[argi+1];
+		used = 2;
+		break;
+        case 'l':  /* YLegend - rewritten by Oliver Haidi, re-rewritten by Jon Barber */
                 { int i, j, k, loop = 1;
+              int start=1, got_esc=0, append_ok;
                   char* qstr;
                   longup = (char *)calloc(1,100);
                   *longup = 0;
@@ -1440,79 +1499,137 @@ char **argv;
                      passing .... or because we don't know
                      better. Under Unix we just would say.  if
                      ((sscanf(argv[argi+1],"[%[^]]]", longup); */
-                  for (i=1; (i<argc) && loop; i++)   /* check all args delimited by '[' and ']' */
-                  { qstr = argv[argi+i];
-                    /* fprintf(stderr, "qstr = \"%s\"\n", qstr); */
-                    for (j=0; ((size_t) j<strlen(qstr)) && loop; j++)   /* check arg */
-                    { if (qstr[j] == '[')   /* beginning of YLegend-string */
-                      { j++;        /* skip '[' */
-                        *longup = 0;   /* clear */
-                      }
-                      if (strlen(qstr+j) >= 2)   /* check for escapes... */
-                      { if (qstr[j] == ']')   /* end of YLegend-string */
-                        { loop = 0;   /* exit */
-                        }
-                        else if (qstr[j] == '\\')   /* escape character '\' found */
-                        { j++;   /* skip '\' */
-                          if (qstr[j+1] == 0)   /* end of qstr reached */
-                          { fprintf(stderr, "Rateup ERROR: YLegend: bad escape \"\\\"! Use \"\\\\\" for \"\\\", \"\\[\" for \"[\" or \"\\]\" for \"]\".\n");
+              /* at start, check 1st char must be [ */
+              if ( argv[argi+1][0] !=  '[' ) {
+                 fprintf(stderr, "Rateup ERROR: YLegend: Option must be passed with '[' at start and  ']' at end (these will not be printed).\n");
                             return(1);
                           }
-                          if ((qstr[j] == '[') ||
-                              (qstr[j] == '\\') ||
-                              (qstr[j] == ']'))
-                          { /* escape okay -> append qstr[j] */
-                            k = strlen(longup);
-                            if ((size_t) (k + 1) > 99)
-                            { fprintf(stderr, "1: Rateup ERROR: YLegend to long ... !\n");
+              for (i=1; (i<argc) && loop; i++) {   /* check all args until unescaped ']'  */ 
+                qstr = argv[argi+i];
+                for (j=start; ((size_t) j<strlen(qstr)) && loop; j++) {
+                  start=0; /* 1st char in 1st arg already checked */ 
+                  append_ok=1; /* OK to append unless we find otherwise */
+                  if ( qstr[j] == '\\' ) {
+                    if  ( ++got_esc == 1 ) {
+                      append_ok = 0; /* don't append 1st '/' */
+                    } else { 
+                      got_esc = 0; /* 2nd '/' in a row, i.e. '//' */
+                    }
+                  }
+                  if ( qstr[j] == ']' ) { /* is this the end? */
+                    if ( got_esc == 1 ) {
+                      if ( strlen(qstr+j) >=2 ) {
+                        append_ok=1; 
+                        got_esc=0;
+                      } else {
+                        fprintf(stderr, "1: rateup ERROR: YLegend:  use \"\\]\" for \"]\" or \"\\\\\" for \"\\\".\n"); 
+                        return(1);
+                      }
+                    } else {
+                      if ( strlen(qstr+j) >=2 ) { 
+                        fprintf(stderr, "2: rateup ERROR: YLegend:  use \"\\]\" for \"]\" or \"\\\\\" for \"\\\".\n"); 
                               return(1);
                             }
-                            longup[k] = qstr[j];
-                            longup[k + 1] = 0;
+                      loop = 0;
+                      append_ok=0;
                           }
-                          else   /* bad escape */
-                          { fprintf(stderr, "2: Rateup ERROR: YLegend: bad escape \"\\%c\"! Use \"\\\\\" for \"\\\", \"\\[\" for \"[\" or \"\\]\" for \"]\".\n", qstr[j]);
-                            return(1);
-                          }
-                        }
-                        else   /* append qstr[j] */
-                        { k = strlen(longup);
-                          if ((size_t) (k + 1) > 99)
-                          { fprintf(stderr, "3: Rateup ERROR: YLegend to long ... !\n");
-                            return(1);
-                          }
-                          longup[k] = qstr[j];
-                          longup[k + 1] = 0;
-                        }
-                      }
-                      else   /* handle last character */
-                      { if (qstr[j] == ']')   /* end of YLegend-string */
-                        { loop = 0;   /* exit */
-                        }
-                        else   /* append last character */
-                        { if ((strlen(longup) + strlen(qstr+j)) > (size_t)99)
-                          { fprintf(stderr, "4: Rateup ERROR: YLegend to long ... !\n");
-                            return(1);
-                          }
-                          strcat(longup, qstr+j);
-                        }
-                      }
-                    }   /* for (j=0; (j<strlen(qstr)) && loop; j++) */
+                  } 
+                  if ( append_ok == 1 ) {
                     k = strlen(longup);
-                    if ((size_t) (k + 1) > 99)
-                    { fprintf(stderr, "5: Rateup ERROR: YLegend to long ... !\n");
+                    if ((size_t) (k + 1) > 99 ) {
+                      fprintf(stderr, "3: rateup ERROR: YLegend too long\n");
+                            return(1);
+                          }
+                    longup[k]=qstr[j];
+                    longup[k+1] = 0;
+                        }
+                } /* for (j=start...)   */
+                /* append space */
+                k = strlen(longup);
+                if ((size_t) (k + 1) > 99 ) { 
+                  fprintf(stderr, "4: rateup ERROR: YLegend too long\n");
+                            return(1);
+                          }
+                longup[k] = ' ' ;
+                longup[k + 1]= 0;
+              } /* for (i =1 ... */
+              used=i;	
+                        }
+            /* remove trailing space */
+            longup[max(0, (signed) strlen(longup)-1)] = 0;
+            shortup = longup;
+            /* fprintf(stderr, "YLegend = \"%s\"\n", longup);  */
+            break; 
+        case 'T':  /* pngTitle - based on YLegend */
+            { int i, j, k, loop = 1;
+              int start=1, got_esc=0, append_ok;
+              char* qstr;
+              pngtitle = (char *)calloc(1,100);
+              *pngtitle = 0;
+              /* this rather twisty argument scanning is necesary
+                 because NT command.coms rather dumb argument
+                 passing .... or because we don't know
+                 better. Under Unix we just would say.  if
+                 ((sscanf(argv[argi+1],"[%[^]]]", pngtitle); */
+              /* at start, check 1st char must be [ */
+              if ( argv[argi+1][0] !=  '[' ) {
+                 fprintf(stderr, "Rateup ERROR: YLegend: Option must be passed with '[' at start and  ']' at end (these will not be printed).\n");
+                 return(1);
+                      }
+              for (i=1; (i<argc) && loop; i++) {   /* check all args until unescaped ']'  */ 
+                qstr = argv[argi+i];
+                for (j=start; ((size_t) j<strlen(qstr)) && loop; j++) {
+                  start=0; /* 1st char in 1st arg already checked */ 
+                  append_ok=1; /* OK to append unless we find otherwise */
+                  if ( qstr[j] == '\\' ) {
+                    if  ( ++got_esc == 1 ) {
+                      append_ok = 0; /* don't append 1st '/' */
+                    } else { 
+                      got_esc = 0; /* 2nd '/' in a row, i.e. '//' */
+                    }
+                  }
+                  if ( qstr[j] == ']' ) { /* is this the end? */
+                    if ( got_esc == 1 ) {
+                      if ( strlen(qstr+j) >=2 ) {
+                        append_ok=1; 
+                        got_esc=0;
+                      } else {
+                        fprintf(stderr, "1: rateup ERROR: YLegend:  use \"\\]\" for \"]\" or \"\\\\\" for \"\\\".\n"); 
+                        return(1);
+                        }
+                    } else {
+                      if ( strlen(qstr+j) >=2 ) { 
+                        fprintf(stderr, "2: rateup ERROR: YLegend:  use \"\\]\" for \"]\" or \"\\\\\" for \"\\\".\n"); 
+                            return(1);
+                          }
+                      loop = 0;
+                      append_ok=0;
+                        }
+                      }
+                  if ( append_ok == 1 ) {
+                    k = strlen(pngtitle);
+                    if ((size_t) (k + 1) > 99 ) {
+                      fprintf(stderr, "3: rateup ERROR: YLegend too long\n");
                       return(1);
                     }
-                    /* strcat (longup," "); */
-                    longup[k] = ' ';   /* append space */
-                    longup[k + 1] = 0;
-                  }   /* for (i=1; (i<argc) && loop; i++) */
-                  used = i;
+                    pngtitle[k]=qstr[j];
+                    pngtitle[k+1] = 0;
                 }
-                /* remove the last space in longup: */
-                longup[max(0, (signed) strlen(longup)-1)] = 0;
-                shortup = longup;
-                /* fprintf(stderr, "YLegend = \"%s\"\n", longup); */
+                } /* for (j=start...)   */
+                /* append space */
+                k = strlen(pngtitle);
+                if ((size_t) (k + 1) > 99 ) { 
+                  fprintf(stderr, "4: rateup ERROR: YLegend too long\n");
+                  return(1);
+                }
+                pngtitle[k] = ' ' ;
+                pngtitle[k + 1]= 0;
+              } /* for (i =1 ... */
+              used=i;	
+            }
+            /* remove trailing space */
+            pngtitle[max(0, (signed) strlen(pngtitle)-1)] = 0;
+            /* fprintf(stderr, "YLegend = \"%s\"\n", pngtitle);  */
                 break;
 	    default:
 	        fprintf(stderr,"Rateup ERROR: Can't cope with %s, sorry!\n",argv[argi]);
@@ -1521,3 +1638,5 @@ char **argv;
     }
     return(0);
 }
+
+
